@@ -47,4 +47,9 @@ class MopacReadError(Exception):
 class SegmentedGraphError(Exception):
     '''
     Thrown by Clustered CSearch when graph has more than one connected component.
+    '''
+
+class NoOrbitalError(Exception):
+    '''
+    Thrown when trying to access orbital data when they are not present
     '''
\ No newline at end of file
